#! /usr/bin/env lua

--[[
	Demonstrates the use of lgui.ProxyList to implement a custom list TreeModel
	in Lua.
--]]

require("lgui")

-- Create the "proxy" table
local MyProxy = {
}

function MyProxy.new()
	local self = {}
	self.cols = {[0] = {}, [1] = {}}
	setmetatable(self, {__index = MyProxy})
	
	return self
end

function MyProxy:iterNext(row)
	if row < 100 then
		return true
	end
end

function MyProxy:getValue(row, col)
	if self.cols[col][row] then
		return self.cols[col][row]
	end
	
	if col == 0 then
		return 100 - (row % 100)
	else
		return "Row " .. row
	end
end

function MyProxy:setValue(row, col, value)
	self.cols[col][row] = value
end

-- Create the GUI

local window = lgui.Window.new()
local scroll = lgui.ScrolledWindow.new()
local proxy = MyProxy.new()
local model = lgui.ProxyList.new(proxy, "gint", "gchararray")
local view = lgui.TreeView.newWithModel(model)
window:connect("delete-event", lgui.quit)

local rend1 = lgui.CellRendererProgress.new()
local rend2 = lgui.CellRendererText.new()
local col1	 = lgui.TreeViewColumn.new("Col 1", rend1, "value", 0)
local col2	 = lgui.TreeViewColumn.new("Col 2", rend2, "text", 1)
col1:set("expand", true)
col2:set("expand", true)
view:appendColumn(col1, col2)

local iter = lgui.TreeIter.new()

function rend2Edited(data, path, new)
	lgui.TreeModel.getIterFromString(model, iter, path)
	model:setValue(iter, 1, new)
end

rend2:set("editable", true)
rend2:connect("edited", rend2Edited)

window:set("window-position", lgui.WIN_POS_CENTER, "title", "Custom ListModel demo",
	"width-request", 400, "height-request", 500)
window:connect("delete-event", lgui.quit)
scroll:add(view)
window:add(scroll)
window:showAll()

lgui.main()
