/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "dialog.h"

static void priv_hook_callback(GtkAboutDialog* about, const gchar* link, Data* data);

/* GtkDialog */

int lgui_dialog_new(lua_State* L)
{
	GtkWidget* ptr = gtk_dialog_new();
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_dialog_add_button(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	int size = lua_gettop(L), i;
	Object* obj = lua_touserdata(L, 1);

	for(i = 2; i <= size; i += 2)
	{
		GtkWidget* ptr = gtk_dialog_add_button(GTK_DIALOG(obj->pointer), lua_tostring(L, i), lua_tonumber(L, i + 1));
		object_new(L, ptr, FALSE);
	}

	return (size - 1) / 2;
}

int lgui_dialog_get_vbox(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	GtkWidget* ptr = GTK_DIALOG(obj->pointer)->vbox;
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_dialog_run(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gint res = gtk_dialog_run(GTK_DIALOG(obj->pointer));
	lua_pushnumber(L, res);

	return 1;
}

int lgui_dialog_response(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_dialog_response(GTK_DIALOG(obj->pointer), lua_tointeger(L, 2));
	
	return 0;
}

/* GtkFileFilter */

int lgui_file_filter_new(lua_State* L)
{
	GtkFileFilter* ptr = gtk_file_filter_new();
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_file_filter_set_name(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);

	Object* obj = lua_touserdata(L, 1);
	gtk_file_filter_set_name(GTK_FILE_FILTER(obj->pointer), lua_tostring(L, 2));

	return 0;
}

int lgui_file_filter_get_name(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	const gchar* name = gtk_file_filter_get_name(GTK_FILE_FILTER(obj->pointer));
	lua_pushstring(L, name);

	return 1;
}

int lgui_file_filter_add_mimetype(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);

	Object* obj = lua_touserdata(L, 1);
	gtk_file_filter_add_mime_type(GTK_FILE_FILTER(obj->pointer), lua_tostring(L, 2));

	return 0;
}

int lgui_file_filter_add_pattern(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);

	Object* obj = lua_touserdata(L, 1);
	gtk_file_filter_add_pattern(GTK_FILE_FILTER(obj->pointer), lua_tostring(L, 2));

	return 0;
}

int lgui_file_filter_add_pixbuf_formats(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	gtk_file_filter_add_pixbuf_formats(GTK_FILE_FILTER(obj->pointer));

	return 0;
}

/* GtkFileChooser */

int lgui_file_chooser_add_filter(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	int size = lua_gettop(L), i;

	Object* obj = lua_touserdata(L, 1);

	for(i = 2; i <= size; i++)
	{
		luaL_checktype(L, i, LUA_TUSERDATA);
		Object* filter = lua_touserdata(L, i);
		gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(obj->pointer), GTK_FILE_FILTER(filter->pointer));
	}

	return 0;
}

int lgui_file_chooser_remove_filter(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	int size = lua_gettop(L), i;

	Object* obj = lua_touserdata(L, 1);

	for(i = 2; i <= size; i++)
	{
		luaL_checktype(L, i, LUA_TUSERDATA);
		Object* filter = lua_touserdata(L, i);
		gtk_file_chooser_remove_filter(GTK_FILE_CHOOSER(obj->pointer), GTK_FILE_FILTER(filter->pointer));
	}

	return 0;
}

int lgui_file_chooser_get_filename(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	gchar* filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(obj->pointer));
	lua_pushstring(L, filename);
	g_free(filename);

	return 1;
}

int lgui_file_chooser_get_filenames(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	GSList* files = gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(obj->pointer));
	lua_pushliteral(L, "__lobjGSList2table");
	lua_rawget(L, LUA_GLOBALSINDEX);
	lua_pushlightuserdata(L, files);
	lua_pushinteger(L, TYPE_STRING);
	lua_call(L, 2, 1);

	g_slist_foreach (files, (GFunc)g_free, NULL);
	g_slist_free (files);

	return 1;
}

int lgui_file_chooser_set_filename(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);

	Object* obj = lua_touserdata(L, 1);
	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(obj->pointer), lua_tostring(L, 2));

	return 0;
}

/* GtkFileChooserDialog */

int lgui_file_chooser_dialog_new(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	luaL_checktype(L, 3, LUA_TNUMBER);
	luaL_checktype(L, 4, LUA_TSTRING);
	luaL_checktype(L, 5, LUA_TNUMBER);
	int i, size = lua_gettop(L);
	
	GtkWindow* parent = NULL;
	
	if(lua_type(L, 2) == LUA_TUSERDATA)
	{		
		Object* obj = lua_touserdata(L, 2);
		parent = GTK_WINDOW(obj->pointer);
	}
	
	GtkWidget* ptr = gtk_file_chooser_dialog_new(lua_tostring(L, 1), parent,
	(GtkFileChooserAction)lua_tointeger(L, 3), lua_tostring(L, 4), lua_tointeger(L, 5), NULL);

	/* The buttons */
	for(i = 6; i <= size; i += 2)
	{
		luaL_checktype(L, i, LUA_TSTRING);
		luaL_checktype(L, i + 1, LUA_TNUMBER);
		gtk_dialog_add_button(GTK_DIALOG(ptr), lua_tostring(L, i), lua_tonumber(L, i + 1));
	}

	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_file_chooser_select_filename(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	Object* obj = lua_touserdata(L, 1);
	
	gboolean res = gtk_file_chooser_select_filename(
		GTK_FILE_CHOOSER(obj->pointer), lua_tostring(L, 2));
		
	lua_pushboolean(L, res);
	return 1;
}

int lgui_file_chooser_unselect_filename(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_file_chooser_unselect_filename(GTK_FILE_CHOOSER(obj->pointer),
		lua_tostring(L, 2));
		
	return 0;
}

int lgui_file_chooser_select_all(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_file_chooser_select_all(GTK_FILE_CHOOSER(obj->pointer));
		
	return 0;
}

int lgui_file_chooser_unselect_all(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_file_chooser_unselect_all(GTK_FILE_CHOOSER(obj->pointer));
		
	return 0;
}

int lgui_file_chooser_set_current_folder(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	Object* obj = lua_touserdata(L, 1);
	
	gboolean res = gtk_file_chooser_set_current_folder(
		GTK_FILE_CHOOSER(obj->pointer), lua_tostring(L, 2));
		
	lua_pushboolean(L, res);
	return 1;
}

int lgui_file_chooser_set_current_name(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_file_chooser_set_current_name( GTK_FILE_CHOOSER(obj->pointer),
		lua_tostring(L, 2));
		
	return 0;
}

int lgui_file_chooser_get_current_folder(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gchar* res = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(obj->pointer));
		
	lua_pushstring(L, res);
	g_free(res);
	
	return 1;
}

/* GtkColorSelectionDialog */

int lgui_color_selection_dialog_new(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	GtkWidget* ptr = gtk_color_selection_dialog_new(lua_tostring(L, 1));
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_color_selection_dialog_get_chooser(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	GtkWidget* ptr = (GTK_COLOR_SELECTION_DIALOG(obj->pointer))->colorsel;
	object_new(L, ptr, FALSE);

	return 1;
}

/* GtkFontSelectionDialog */

int lgui_font_selection_dialog_new(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	GtkWidget* ptr = gtk_font_selection_dialog_new(lua_tostring(L, 1));
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_font_selection_dialog_get_chooser(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	GtkFontSelection* ptr = GTK_FONT_SELECTION(GTK_FONT_SELECTION_DIALOG(obj->pointer)->fontsel);
	object_new(L, ptr, FALSE);

	return 1;
}

/* GtkAboutDialog */

int lgui_about_dialog_new(lua_State* L)
{
	GtkWidget* ptr = gtk_about_dialog_new();
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_about_dialog_set_email_hook(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TFUNCTION);

	int size = lua_gettop(L);
	int ud_ref = -1;
	/* User data? */
	if(size > 1)
	{
		luaL_checkany(L, 2);

		/* Create a reference to the user data */
		ud_ref = luaL_ref(L, LUA_REGISTRYINDEX);
	}

	lua_pushvalue(L, 1);
	int function_ref = luaL_ref(L, LUA_REGISTRYINDEX);

	Data* data = g_malloc(sizeof(Data));
	data->L = L;
	data->function_ref = function_ref;
	data->ud_ref = ud_ref;

	gtk_about_dialog_set_email_hook((GtkAboutDialogActivateLinkFunc)priv_hook_callback,
		data, callback_free);

	#ifdef IDEBUG
	g_debug("Email hook setted!");
	#endif

	return 0;
}

int lgui_about_dialog_set_url_hook(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TFUNCTION);

	int size = lua_gettop(L);
	int ud_ref = -1;

	/* User data? */
	if(size > 1)
	{
		luaL_checkany(L, 2);

		/* Create a reference to the user data */
		ud_ref = luaL_ref(L, LUA_REGISTRYINDEX);
	}

	lua_pushvalue(L, 1);
	int function_ref = luaL_ref(L, LUA_REGISTRYINDEX);

	Data* data = g_malloc(sizeof(Data));
	data->L = L;
	data->function_ref = function_ref;
	data->ud_ref = ud_ref;

	gtk_about_dialog_set_url_hook((GtkAboutDialogActivateLinkFunc)priv_hook_callback,
		data, callback_free);

	#ifdef IDEBUG
	g_debug("URL hook setted!");
	#endif

	return 0;
}

static void priv_hook_callback(GtkAboutDialog* about, const gchar* link, Data* data)
{
	lua_State* L = data->L;
	int size = lua_gettop(L);

	/* Push the function and user data */
	lua_rawgeti(L, LUA_REGISTRYINDEX, data->function_ref);
	lua_rawgeti(L, LUA_REGISTRYINDEX, data->ud_ref);

	/* Push the link */
	lua_pushstring(L, link);
	
	/* Call the function */
	lua_call(L, 2, 0);
	lua_settop(L, size);

	#ifdef IDEBUG
	g_debug("Hook returning!");
	#endif
}
