/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "entry.h"

/* GtkEntryCompletion */

int lgui_entry_completion_new(lua_State* L)
{
	GtkEntryCompletion* ptr = gtk_entry_completion_new();
	object_new(L, ptr, TRUE);

	return 1;
}

int lgui_entry_completion_set_model(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	Object* model = lua_touserdata(L, 2);

	gtk_entry_completion_set_model(GTK_ENTRY_COMPLETION(obj->pointer),
	GTK_TREE_MODEL(model->pointer));

	return 0;
}

int lgui_entry_completion_set_text_column(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);

	gtk_entry_completion_set_text_column(GTK_ENTRY_COMPLETION(obj->pointer),
	lua_tointeger(L, 2));

	return 0;
}

/* GtkEntry */

int lgui_entry_new(lua_State* L)
{
	GtkWidget* ptr = gtk_entry_new();
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_entry_set_completion(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	Object* comp = lua_touserdata(L, 2);
	gtk_entry_set_completion(GTK_ENTRY(obj->pointer), GTK_ENTRY_COMPLETION(comp->pointer));

	return 0;
}

/* GtkSpinButton */

int lgui_spin_button_new(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	
	GtkWidget* ptr = gtk_spin_button_new(GTK_ADJUSTMENT(obj->pointer), lua_tointeger(L, 2), lua_tointeger(L, 3));
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_spin_button_new_with_range(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TNUMBER);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TNUMBER);
	
	GtkWidget* ptr = gtk_spin_button_new_with_range(lua_tointeger(L, 1), lua_tointeger(L, 2), lua_tointeger(L, 3));
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_spin_button_update(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_spin_button_update(GTK_SPIN_BUTTON(obj->pointer));
	
	return 0;
}
