/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "iconview.h"

int lgui_icon_view_new(lua_State* L)
{
	GtkWidget* ptr = gtk_icon_view_new();
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_icon_view_new_with_model(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	GtkWidget* ptr = gtk_icon_view_new_with_model(GTK_TREE_MODEL(obj->pointer));
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_icon_view_get_selected_items(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	GList* list = gtk_icon_view_get_selected_items(GTK_ICON_VIEW(obj->pointer));

	lua_pushliteral(L, "__lobjGList2table");
	lua_rawget(L, LUA_GLOBALSINDEX);
	lua_pushlightuserdata(L, list);
	lua_pushinteger(L, TYPE_CUSTOM);
	lua_pushcfunction(L, lgui_tree_path_handler);
	lua_call(L, 3, 1);

	/* Free the list */
	g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
	g_list_free(list);

	return 1;
}

int lgui_icon_view_get_path_at_pos(lua_State* L)
{
		luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TNUMBER);
	
	Object* obj = lua_touserdata(L, 1);
	
	GtkTreePath* path = gtk_icon_view_get_path_at_pos(GTK_ICON_VIEW(obj->pointer),
		lua_tointeger(L, 2), lua_tointeger(L, 3));
	
	if(path == NULL)
		return 0;
	
	gchar* spath = gtk_tree_path_to_string(path);
	
	lua_pushstring(L, spath);
	gtk_tree_path_free(path);
	g_free(spath);
	
	return 1;
}

int lgui_icon_view_convert_widget_to_bin_window_coords(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA); 
	luaL_checktype(L, 2, LUA_TNUMBER); 
	luaL_checktype(L, 3, LUA_TNUMBER);
	
	Object* obj = lua_touserdata(L, 1);
	gint nX, nY;
	
	gtk_icon_view_convert_widget_to_bin_window_coords(GTK_ICON_VIEW(obj->pointer),
		lua_tointeger(L, 2), lua_tointeger(L, 3), &nX, &nY);
		
	lua_pushinteger(L, nX);
	lua_pushinteger(L, nY);
	
	return 2; 
}

int lgui_icon_view_select_all(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_icon_view_select_all(GTK_ICON_VIEW(obj->pointer));
	
	return 0;
}

int lgui_icon_view_unselect_all(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_icon_view_unselect_all(GTK_ICON_VIEW(obj->pointer));
	
	return 0;
}
