/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "print.h"

/* GtkPrintSettings */

static void priv_foreach_callback(const gchar* key, const gchar* value, lua_State* L)
{
	int top = lua_gettop(L);
	
	/* Push the function and userdata */
	lua_pushvalue(L, -2);
	lua_pushvalue(L, -2);
	
	/* Push the key and value */
	lua_pushstring(L, key);
	lua_pushstring(L, value);

	/* Call the function */
	lua_call(L, 3, 1);
	
	lua_settop(L, top);

	#ifdef IDEBUG
	g_debug("ForeachCallback returning!");
	#endif
}

int lgui_print_settings_new(lua_State* L)
{
	GtkPrintSettings* ptr = gtk_print_settings_new();
	object_new(L, ptr, TRUE);
	
	return 1;
}

int lgui_print_settings_copy(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	GtkPrintSettings* ptr = gtk_print_settings_copy(GTK_PRINT_SETTINGS(obj->pointer));
	object_new(L, ptr, TRUE);
	
	return 1;
}

int lgui_print_settings_get(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	Object* obj = lua_touserdata(L, 1);
	
	lua_pushstring(L, gtk_print_settings_get(GTK_PRINT_SETTINGS(obj->pointer),
		lua_tostring(L, 2)));
	
	return 1;
}

int lgui_print_settings_set(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	
	Object* obj = lua_touserdata(L, 1);
	
	if(lua_isnoneornil(L, 3))
		gtk_print_settings_unset(GTK_PRINT_SETTINGS(obj->pointer), lua_tostring(L, 2));
	else
	{
		luaL_checktype(L, 3, LUA_TSTRING);
		gtk_print_settings_set(GTK_PRINT_SETTINGS(obj->pointer),
			lua_tostring(L, 2), lua_tostring(L, 3));
	}
	
	return 0;
}

int lgui_print_settings_foreach(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TFUNCTION);
	
	Object* obj = lua_touserdata(L, 1);
	lua_pushvalue(L, 2);
	lua_pushvalue(L, 3);

	gtk_print_settings_foreach(GTK_PRINT_SETTINGS(obj->pointer), 
		(GtkPrintSettingsFunc)priv_foreach_callback, L);

	return 0;
}

/* GtkPaperSize */

static int priv_paper_size_gc(lua_State* L)
{
	#ifdef IDEBUG
	g_debug("Garbage collecting a PaperSize!");
	#endif
	
	Object* obj = lua_touserdata(L, 1);
	if(obj->need_unref) gtk_paper_size_free(obj->pointer);

	return 0;
}

static int priv_paper_size_tostring(lua_State* L)
{	
	Object* obj = lua_touserdata(L, 1);
	lua_pushstring(L, gtk_paper_size_get_display_name(obj->pointer));
	
	return 1;
}

static void priv_push_paper_size(lua_State* L, GtkPaperSize* ptr, gboolean need_unref)
{
	#ifdef IDEBUG
	g_debug("Creating a new PaperSize!");
	#endif
	
	if(ptr)
	{
		Object* obj = lua_newuserdata(L, sizeof(Object));
		obj->pointer = ptr;
		obj->need_unref = need_unref;
		int top = lua_gettop(L);
		
		/* Shared metatable */
		if(luaL_newmetatable(L, "__lguiPaPeRMt"))
		{
			lua_getfield(L, LUA_GLOBALSINDEX, "__lobjRegisterSpecial");
			lua_pushvalue(L, -2);
			lua_pushliteral(L, "lgui");
			lua_pushliteral(L, "PaperSize");
			lua_pushcfunction(L, priv_paper_size_gc);
			lua_pushcfunction(L, priv_paper_size_tostring);
			lua_call(L, 5, 0);
		}

		/* Set the object metatable */
		lua_setmetatable(L, top);
	}
	else
		lua_pushnil(L);
}

int lgui_paper_size_new(lua_State* L)
{
	GtkPaperSize* ptr = gtk_paper_size_new(lua_tostring(L, 1));
	priv_push_paper_size(L, ptr, TRUE);
	
	return 1;
}

int lgui_paper_size_new_custom(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	luaL_checktype(L, 2, LUA_TSTRING);
	luaL_checktype(L, 3, LUA_TNUMBER);
	luaL_checktype(L, 4, LUA_TNUMBER);
	luaL_checktype(L, 5, LUA_TNUMBER);
	
	GtkPaperSize* ptr = gtk_paper_size_new_custom(lua_tostring(L, 1),
		lua_tostring(L, 2), lua_tonumber(L, 3), lua_tonumber(L, 4), 
		lua_tointeger(L, 5));
		
	priv_push_paper_size(L, ptr, TRUE);
	
	return 1;
}

int lgui_paper_size_get(lua_State* L) /* All-in-one getter */
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	
	Object* obj = lua_touserdata(L, 1);
	GtkPaperSize* ptr = obj->pointer;
	GtkUnit unit = lua_tointeger(L, 2);
	
	lua_newtable(L);
	int top = lua_gettop(L);
	
	lua_pushstring(L, gtk_paper_size_get_name(ptr));
	lua_setfield(L, top, "name");
	
	lua_pushstring(L, gtk_paper_size_get_display_name(ptr));
	lua_setfield(L, top, "display-name");
	
	lua_pushstring(L, gtk_paper_size_get_ppd_name(ptr));
	lua_setfield(L, top, "ppd-name");
	
	lua_pushnumber(L, gtk_paper_size_get_width(ptr, unit));
	lua_setfield(L, top, "width");
	
	lua_pushnumber(L, gtk_paper_size_get_height(ptr, unit));
	lua_setfield(L, top, "height");
	
	lua_pushboolean(L, gtk_paper_size_is_custom(ptr));
	lua_setfield(L, top, "is-custom");
	
	lua_pushboolean(L, gtk_paper_size_get_default_bottom_margin(ptr, unit));
	lua_setfield(L, top, "default-botton-margin");
	
	lua_pushboolean(L, gtk_paper_size_get_default_top_margin(ptr, unit));
	lua_setfield(L, top, "default-top-margin");
	
	lua_pushboolean(L, gtk_paper_size_get_default_left_margin(ptr, unit));
	lua_setfield(L, top, "default-left-margin");
	
	lua_pushboolean(L, gtk_paper_size_get_default_right_margin(ptr, unit));
	lua_setfield(L, top, "default-right-margin");
	
	lua_settop(L, top);
	
	return 1;
}

/* GtkPageSetup */

int lgui_page_setup_new(lua_State* L)
{
	GtkPageSetup* ptr = gtk_page_setup_new();
	object_new(L, ptr, TRUE);
	
	return 1;
}

int lgui_page_setup_copy(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	GtkPageSetup* ptr = gtk_page_setup_copy(GTK_PAGE_SETUP(obj->pointer));
	object_new(L, ptr, TRUE);
	
	return 1;
}

int lgui_page_setup_set(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TTABLE);
	
	Object* obj = lua_touserdata(L, 1);
	GtkPageSetup* setup = GTK_PAGE_SETUP(obj->pointer);
	GtkUnit unit = lua_tointeger(L, 2);
	
	lua_getfield(L, 3, "orientation");
	if(lua_isnumber(L, -1)) gtk_page_setup_set_orientation(setup, lua_tonumber(L, -1));
	
	lua_getfield(L, 3, "paper-size");
	if(lua_isuserdata(L, -1))
	{
		GtkPaperSize* tmp = ((Object*)lua_touserdata(L, -1))->pointer;
		gtk_page_setup_set_paper_size(setup, tmp);
	}
	
	lua_getfield(L, 3, "paper-size-and-default-margins");
	if(lua_isuserdata(L, -1))
	{
		GtkPaperSize* tmp = ((Object*)lua_touserdata(L, -1))->pointer;
		gtk_page_setup_set_paper_size_and_default_margins(setup, tmp);
	}
	
	lua_getfield(L, 3, "top-margin");
	if(lua_isnumber(L, -1)) gtk_page_setup_set_top_margin(setup, lua_tonumber(L, -1), unit);
	
	lua_getfield(L, 3, "botton-margin");
	if(lua_isnumber(L, -1)) gtk_page_setup_set_bottom_margin(setup, lua_tonumber(L, -1), unit);
	
	lua_getfield(L, 3, "left-margin");
	if(lua_isnumber(L, -1)) gtk_page_setup_set_left_margin(setup, lua_tonumber(L, -1), unit);
	
	lua_getfield(L, 3, "right-margin");
	if(lua_isnumber(L, -1)) gtk_page_setup_set_right_margin(setup, lua_tonumber(L, -1), unit);
	
	return 0;
}

int lgui_page_setup_get(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	Object* obj = lua_touserdata(L, 1);
	GtkPageSetup* setup = GTK_PAGE_SETUP(obj->pointer);
	GtkUnit unit = lua_tointeger(L, 2);
	
	lua_newtable(L);
	int top = lua_gettop(L);
	
	lua_pushinteger(L, gtk_page_setup_get_orientation(setup));
	lua_setfield(L, top, "orientation"); 
	
	priv_push_paper_size(L, gtk_page_setup_get_paper_size(setup), FALSE);
	lua_setfield(L, top, "paper-size"); 
	
	lua_pushinteger(L, gtk_page_setup_get_top_margin(setup, unit));
	lua_setfield(L, top, "top-margin"); 
	
	lua_pushinteger(L, gtk_page_setup_get_bottom_margin(setup, unit));
	lua_setfield(L, top, "botton-margin"); 
	
	lua_pushinteger(L, gtk_page_setup_get_left_margin(setup, unit));
	lua_setfield(L, top, "left-margin"); 
	
	lua_pushinteger(L, gtk_page_setup_get_right_margin(setup, unit));
	lua_setfield(L, top, "right-margin"); 
	
	lua_pushinteger(L, gtk_page_setup_get_page_width(setup, unit));
	lua_setfield(L, top, "page-width");
	
	lua_pushinteger(L, gtk_page_setup_get_page_height(setup, unit));
	lua_setfield(L, top, "page-height");
	
	lua_pushinteger(L, gtk_page_setup_get_paper_width(setup, unit));
	lua_setfield(L, top, "paper-width");
	
	lua_pushinteger(L, gtk_page_setup_get_paper_height(setup, unit));
	lua_setfield(L, top, "paper-height");
	
	lua_settop(L, top);
	
	return 1;
}

/* GtkPrintOperation */

int lgui_print_operation_new(lua_State* L)
{
	GtkPrintOperation* ptr = gtk_print_operation_new();
	object_new(L, ptr, TRUE);

	return 1;
}

int lgui_print_operation_run(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);

	int top = lua_gettop(L);
	GtkWindow* parent = NULL;

	if(top >= 3)
	{
		Object* obj = lua_touserdata(L, 3);
		luaL_checktype(L, 3, LUA_TUSERDATA);
		parent = GTK_WINDOW(obj->pointer);
	}
	
	GError* errors = NULL;	
	Object* obj = lua_touserdata(L, 1);
	
	int res = (int)gtk_print_operation_run(GTK_PRINT_OPERATION(obj->pointer),
	(GtkPrintOperationAction)lua_tointeger(L, 2), parent, &errors);

	if(errors)
	{
		lua_pushboolean(L, FALSE);
		lua_pushinteger(L, errors->code);
		lua_pushstring(L, errors->message);
		g_error_free(errors);
		return 3;
	}

	lua_pushnumber(L, res);

	return 1;
}

int lgui_print_operation_get_error(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	GError* errors = NULL;
	
	gtk_print_operation_get_error(GTK_PRINT_OPERATION(obj->pointer), &errors);
	
	if(errors)
	{
		lua_pushinteger(L, errors->code);
		lua_pushstring(L, errors->message);
		g_error_free(errors);
		return 2;
	}
	
	return 0;
}

int lgui_print_run_page_setup_dialog(lua_State* L)
{
	luaL_checktype(L, 3, LUA_TUSERDATA);
	
	GtkPageSetup* setup = NULL;
	GtkWindow* parent = NULL;
	
	if(lua_type(L, 1) == LUA_TUSERDATA)
	{
		Object* obj = lua_touserdata(L, 1);
		setup = GTK_PAGE_SETUP(obj->pointer);
	}
	
	if(lua_type(L, 2) == LUA_TUSERDATA)
	{
		Object* obj = lua_touserdata(L, 2);
		parent = GTK_WINDOW(obj->pointer);
	}
	
	Object* obj3 = lua_touserdata(L, 3);
	
	GtkPageSetup* ptr = gtk_print_run_page_setup_dialog(parent, 
		setup, GTK_PRINT_SETTINGS(obj3->pointer));

	object_new(L, ptr, TRUE);
	
	return 1;
}

/* GtkPrintContext */

int lgui_print_context_create_pango_layout(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	PangoLayout* ptr = gtk_print_context_create_pango_layout(GTK_PRINT_CONTEXT(obj->pointer));
	object_new(L, ptr, TRUE);

	return 1;
}

int lgui_print_context_get_size(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	lua_pushnumber(L, gtk_print_context_get_width(GTK_PRINT_CONTEXT(obj->pointer)));
	lua_pushnumber(L, gtk_print_context_get_height(GTK_PRINT_CONTEXT(obj->pointer)));

	return 2;
}

int lgui_print_context_get_dpi(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	lua_pushnumber(L, gtk_print_context_get_dpi_x(GTK_PRINT_CONTEXT(obj->pointer)));
	lua_pushnumber(L, gtk_print_context_get_dpi_y(GTK_PRINT_CONTEXT(obj->pointer)));

	return 2;
}

/* GtkPrintOperationPreview */

int lgui_print_operation_preview_end_preview(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_print_operation_preview_end_preview(GTK_PRINT_OPERATION_PREVIEW(obj->pointer));
	
	return 0;
}

int lgui_print_operation_preview_is_selected(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	Object* obj = lua_touserdata(L, 1);
	
	gboolean res = gtk_print_operation_preview_is_selected(
		GTK_PRINT_OPERATION_PREVIEW(obj->pointer), lua_tonumber(L, 2));
	
	lua_pushboolean(L, res);
	return 1;
}

int lgui_print_operation_preview_render_page(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_print_operation_preview_render_page(
		GTK_PRINT_OPERATION_PREVIEW(obj->pointer), lua_tonumber(L, 2));
	
	return 0;
}
