/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/


#include "treemodelfilter.h"

/* Pre-definition */
static gboolean priv_visible_callback(GtkTreeModel* model, GtkTreeIter* iter, Data* data);

int lgui_tree_model_filter_new(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	int size = lua_gettop(L);

	Object* obj = lua_touserdata(L, 1);
	GtkTreePath* root = NULL;

	if(size > 1)
	{
		Object* path = lua_touserdata(L, 2);
		root = (GtkTreePath*)path->pointer;
	}

	GtkTreeModel* model = gtk_tree_model_filter_new(GTK_TREE_MODEL(obj->pointer), root);
	object_new(L, model, TRUE);

	return 1;
}

int lgui_tree_model_filter_set_visible_function(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TFUNCTION);

	int size = lua_gettop(L);
	int ud_ref = -1;
	Object* obj = lua_touserdata(L, 1);

	/* User data? */
	if(size > 2)
	{
		/* Create a reference to the user data */
		ud_ref = luaL_ref(L, LUA_REGISTRYINDEX);
	}

	lua_pushvalue(L, 2);
	int function_ref = luaL_ref(L, LUA_REGISTRYINDEX);

	Data* data = g_malloc(sizeof(Data));
	data->L = L;
	data->function_ref = function_ref;
	data->ud_ref = ud_ref;

	gtk_tree_model_filter_set_visible_func(GTK_TREE_MODEL_FILTER(obj->pointer),
	(GtkTreeModelFilterVisibleFunc)priv_visible_callback, data, callback_free);

	#ifdef IDEBUG
	g_debug("VisibleFunc setted!");
	#endif

	return 0;
}

gboolean priv_visible_callback(GtkTreeModel *model, GtkTreeIter *iter, Data* data)
{
	lua_State* L = data->L;
	int size = lua_gettop(L);

	/* Push the function and user data */
	lua_rawgeti(L, LUA_REGISTRYINDEX, data->function_ref);
	lua_rawgeti(L, LUA_REGISTRYINDEX, data->ud_ref);

	/* Push the iter */
	struct_new(L, iter, FALSE);

	/* Call the function */
	lua_call(L, 2, 1);

	/* Get the return */
	gboolean res = lua_toboolean(L, -1);
	lua_settop(L, size);

	#ifdef IDEBUG
	g_debug("VisibleCallback returning!");
	#endif

	return res;
}

int lgui_tree_model_filter_set_visible_column(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	gtk_tree_model_filter_set_visible_column(GTK_TREE_MODEL_FILTER(obj->pointer), lua_tointeger(L, 2));

	return 0;
}

int lgui_tree_model_filter_refilter(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	gtk_tree_model_filter_refilter(GTK_TREE_MODEL_FILTER(obj->pointer));

	return 0;
}
