/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "window.h"

int lgui_window_new(lua_State* L)
{
	GtkWidget* ptr = NULL;

	/* User passed the type */
	if(lua_gettop(L) >= 1)
	{
		luaL_checktype(L, 1, LUA_TNUMBER);
		GtkWindowType type = (GtkWindowType)lua_tointeger(L, 1);
		ptr = gtk_window_new(type);
	}
	/* Default */
	else
	{
		ptr = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	}

	object_new(L, ptr, FALSE);
	
	return 1;
}

int lgui_window_add_accel_group(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	gtk_window_add_accel_group(GTK_WINDOW(obj1->pointer), GTK_ACCEL_GROUP(obj2->pointer));

	return 0;
}

int lgui_window_remove_accel_group(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	gtk_window_remove_accel_group(GTK_WINDOW(obj1->pointer), GTK_ACCEL_GROUP(obj2->pointer));

	return 0;
}

int lgui_window_maximize(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	gtk_window_maximize(GTK_WINDOW(obj->pointer));

	return 0;
}

int lgui_window_unmaximize(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	gtk_window_unmaximize(GTK_WINDOW(obj->pointer));

	return 0;
}

int lgui_window_fullscreen(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	gtk_window_fullscreen(GTK_WINDOW(obj->pointer));

	return 0;
}

int lgui_window_unfullscreen(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	gtk_window_unfullscreen(GTK_WINDOW(obj->pointer));

	return 0;
}

int lgui_window_iconify(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	gtk_window_iconify(GTK_WINDOW(obj->pointer));

	return 0;
}

int lgui_window_deiconify(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	gtk_window_deiconify(GTK_WINDOW(obj->pointer));

	return 0;
}

int lgui_window_destroy(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	gtk_widget_destroy(GTK_WIDGET(obj->pointer));
	obj->need_unref = FALSE;
	
	return 0;
}

int lgui_window_resize(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TNUMBER);
	
	Object* obj = lua_touserdata(L, 1);	
	gtk_window_resize(GTK_WINDOW(obj->pointer), lua_tointeger(L, 2), lua_tointeger(L, 3));
		
	return 0;
}

int lgui_window_move(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TNUMBER);
	
	Object* obj = lua_touserdata(L, 1);
	gtk_window_move(GTK_WINDOW(obj->pointer), lua_tointeger(L, 2), lua_tointeger(L, 3));
	
	return 0;
}

int lgui_window_present(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	gtk_window_present(GTK_WINDOW(obj->pointer));
	
	return 0;
}
